@@ -4,10 +4,10 @@ This module contains various functions to plot graphs and algorithms.
 """
 
 __author__ = 'Henry Robbins'
-__all__ = ['plot_tour', 'plot_tree', 'plot_dijkstras', 'plot_mst_algorithm',
-           'plot_tsp_heuristic', 'plot_etching_tour', 'plot_create_tour',
-           'plot_create_spanning_tree', 'plot_assisted_mst_algorithm',
-           'plot_assisted_dijkstras']
+__all__ = ['tour_plot', 'tree_plot', 'dijkstras_plot', 'mst_algorithm_plot',
+           'tsp_heuristic_plot', 'create_tour_plot',
+           'create_spanning_tree_plot', 'assisted_mst_algorithm_plot',
+           'assisted_dijkstras_plot']
 
 
 import numpy as np
@@ -15,13 +15,12 @@ import pandas as pd
 import networkx as nx
 import warnings
 from PIL import Image
-from pkg_resources import resource_stream
+import pkgutil
 from typing import List, Tuple, Dict, Union
 from .algorithms import (dijkstras, prims, kruskals, reverse_kruskals,
                          spanning_tree_cost, neighbor, insertion, two_opt,
-                         tour_cost, Tree)
+                         tour_cost)
 from bokeh.plotting import figure, Figure
-from bokeh.io import show
 from bokeh.models.widgets.markups import Div
 from bokeh.models.widgets.tables import TableColumn, DataTable
 from bokeh.models.renderers import GlyphRenderer
@@ -49,16 +48,21 @@ TERTIARY_DARK_COLOR = '#404040'
 NODE_SIZE = 11
 NODE_LINE_WIDTH = 3
 LINE_WIDTH = 5
+PLOT_MARGIN = 0.085
+# Order: image, underlay, glyph, guide, annotation, overlay
+LABEL_LEVEL = 'annotation'
+LABEL_BACKGROUND_LEVEL = 'guide'
+NODE_LEVEL = 'glyph'
+EDGE_LEVEL = 'underlay'
+IMAGE_LEVEL = 'image'
 
-CONSTANTS_JS = resource_stream('vinal.resources',
-                               'constants.js').read().decode("utf-8")
-PLOT_CREATE_JS = (CONSTANTS_JS
-                  + resource_stream('vinal.resources',
-                                    'plot_create.js').read().decode("utf-8"))
-PLOT_GRAPH_ITERATIONS_JS = (CONSTANTS_JS
-                            + resource_stream('vinal.resources',
-                                              'plot_graph_iterations.js'
-                                              ).read().decode("utf-8"))
+CONSTANTS_JS = pkgutil.get_data(__name__, "resources/constants.js").decode()
+
+tmp = pkgutil.get_data(__name__, "resources/plot_create.js")
+PLOT_CREATE_JS = CONSTANTS_JS + tmp.decode()
+
+tmp = pkgutil.get_data(__name__, "resources/plot_graph_iterations.js")
+PLOT_GRAPH_ITERATIONS_JS = CONSTANTS_JS + tmp.decode()
 
 
 # ------------------------
@@ -77,10 +81,10 @@ def _graph_range(x:List[float], y:List[float]) -> List[float]:
         List[float]: x-range (min_x, max_x) and y-range (min_y, max_y).
     """
     min_x, max_x = min(x), max(x)
-    x_margin = 0.085*(max_x - min_x)
+    x_margin = PLOT_MARGIN * (max_x - min_x)
     min_x, max_x = min_x - x_margin, max_x + x_margin
     min_y, max_y = min(y), max(y)
-    y_margin = 0.085*(max_y - min_y)
+    y_margin = PLOT_MARGIN * (max_y - min_y)
     min_y, max_y = min_y - y_margin, max_y + y_margin
     return (min_x, max_x, min_y, max_y)
 
@@ -88,35 +92,38 @@ def _graph_range(x:List[float], y:List[float]) -> List[float]:
 def _blank_plot(G:nx.Graph,
                 width:int = None,
                 height:int = None,
+                x_range:List[float] = None,
+                y_range:List[float] = None,
                 image:str = None, **kw) -> Figure:
     """Return a blank bokeh plot.
 
-    If an image is provided, the dimensions of the image form the x and y range
-    of the plot. Otherwise, these are computed to make all nodes visible.
+    The x and y axis ranges are chosen from x_range and y_range first. If one
+    or both are not specified then, it defaults to make all nodes of G visible
+    or uses the dimensions of the image if an image was provided.
 
     Args:
         G (nx.Graph): Graph to be plotted on this blank plot.
         width (int, optional): Plot width. Defaults to None.
         height (int, optional): Plot height. Defaults to None.
+        x_range (List[float], optional): Range of x-axis (min_x, max_x).
+        y_range (List[float], optional): Range of y-axis (min_y, max_y).
         image (str, optional): Path to image file. Defaults to None.
 
     Returns:
         Figure: Blank bokeh plot.
     """
-    if image is not None:
-        im = Image.open(image)
-        max_x, max_y = im.size
-        min_x, min_y = 0,0
+    if x_range is not None and y_range is not None:
+        min_x, max_x = x_range
+        min_y, max_y = y_range
     else:
-        if 'x_start' in list(G.nodes[0]):
-            x = (list(nx.get_node_attributes(G,'x_start').values()) +
-                 list(nx.get_node_attributes(G,'x_end').values()))
-            y = (list(nx.get_node_attributes(G,'y_start').values()) +
-                 list(nx.get_node_attributes(G,'y_end').values()))
+        if image is not None:
+            im = Image.open(image)
+            max_x, max_y = im.size
+            min_x, min_y = 0,0
         else:
             x = nx.get_node_attributes(G,'x').values()
             y = nx.get_node_attributes(G,'y').values()
-        min_x, max_x, min_y, max_y = _graph_range(x,y)
+            min_x, max_x, min_y, max_y = _graph_range(x,y)
     plot = figure(x_range=(min_x, max_x),
                   y_range=(min_y, max_y),
                   title="",
@@ -150,7 +157,7 @@ def _add_image(plot:Figure, image:str):
                    y=plot.y_range.end,
                    w=plot.x_range.end - plot.x_range.start,
                    h=plot.y_range.end - plot.y_range.start,
-                   level='image')
+                   level=IMAGE_LEVEL)
 
 
 def _edge_positions(G:nx.Graph,
@@ -240,7 +247,7 @@ def _add_nodes(G:nx.Graph,
     nodes_df = pd.DataFrame([G.nodes[u] for u in sorted(G.nodes())])
     nodes_src = ColumnDataSource(data=nodes_df.to_dict(orient='list'))
 
-    nodes_glyph = plot.circle(x='x', y='y', size=NODE_SIZE, level='glyph',
+    nodes_glyph = plot.circle(x='x', y='y', size=NODE_SIZE, level=NODE_LEVEL,
                               line_color='line_color', fill_color='fill_color',
                               line_width='line_width',
                               nonselection_fill_alpha=1,
@@ -277,24 +284,65 @@ def _add_edges(G:nx.Graph,
                                   hover_line_color=hover_line_color,
                                   line_width=LINE_WIDTH,
                                   nonselection_line_alpha=1,
-                                  level='image',
+                                  level=EDGE_LEVEL,
                                   source=edges_src)
 
     if show_labels:
-        lbl_x = [np.mean(i) for i in nx.get_edge_attributes(G, 'xs').values()]
-        lbl_y = [np.mean(i) for i in nx.get_edge_attributes(G, 'ys').values()]
-        lbl_txt = list(nx.get_edge_attributes(G,'weight').values())
-        labels_src = ColumnDataSource(data={'x': lbl_x,
-                                            'y': lbl_y,
-                                            'text': lbl_txt})
-
-        labels = LabelSet(x='x', y='y', text='text', render_mode='canvas',
-                          source=labels_src)
-        plot.add_layout(labels)
+        _add_labels(G, plot)
 
     return edges_src, edges_glyph
 
 
+def _add_labels(G:nx.Graph, plot:Figure):
+    """Add labels from G to the plot.
+
+    Args:
+        G (nx.Graph): Networkx graph.
+        plot (Figure): Plot to add the labels to.
+    """
+    text = [round(w,2) for w in nx.get_edge_attributes(G,'weight').values()]
+    xs = np.array(list(nx.get_edge_attributes(G, 'xs').values()))
+    ys = np.array(list(nx.get_edge_attributes(G, 'ys').values()))
+
+    x_range = (plot.x_range.end - plot.x_range.start)
+    y_range = (plot.y_range.end - plot.y_range.start)
+    margin_shift_x = (x_range / (2 * PLOT_MARGIN + 1)) * PLOT_MARGIN
+    margin_shift_y = (y_range / (2 * PLOT_MARGIN + 1)) * PLOT_MARGIN
+    x_scale = plot.plot_width / x_range
+    y_scale = plot.plot_height / y_range
+
+    xs = (xs + margin_shift_x - plot.x_range.start) * x_scale
+    ys = (ys + margin_shift_y - plot.y_range.start) * y_scale
+
+    x = np.mean(xs, axis=1)
+    y = np.mean(ys, axis=1)
+    centers = np.vstack((x,y)).T
+    v = np.vstack((xs[:,1] - xs[:,0], ys[:,1] - ys[:,0])).T
+    n = np.divide(v.T, np.linalg.norm(v, axis=1)).T
+
+    lbl_size = np.array([len(str(w)) for w in text]) * 3
+    tmp = np.multiply(n, lbl_size[:, np.newaxis])
+    blank_start = centers - (n * 3 + tmp)
+    blank_end = centers + (n * 3 + tmp)
+    blank_xs = np.vstack((blank_start[:,0], blank_end[:,0])).T
+    blank_ys = np.vstack((blank_start[:,1], blank_end[:,1])).T
+
+    x = (x / x_scale) - margin_shift_x + plot.x_range.start
+    y = (y / y_scale) - margin_shift_y + plot.y_range.start
+    blank_xs = (blank_xs / x_scale) - margin_shift_x + plot.x_range.start
+    blank_ys = (blank_ys / y_scale) - margin_shift_y + plot.y_range.start
+
+    plot.multi_line(xs=blank_xs.tolist(), ys=blank_ys.tolist(),
+                    line_color='white', line_width=LINE_WIDTH+1,
+                    nonselection_line_alpha=1, level=LABEL_BACKGROUND_LEVEL)
+
+    labels_src = ColumnDataSource(data={'x': x, 'y': y, 'text': text})
+    labels = LabelSet(x='x', y='y', text='text', text_align='center',
+                      text_baseline='middle', text_font_size='13px',
+                      text_color='black', level=LABEL_LEVEL, source=labels_src)
+    plot.add_layout(labels)
+
+
 def _get_create_divs(plot:Figure,
                      cost_txt:str = '0.0',
                      click_txt:str = '[]') -> Tuple[Div]:
@@ -363,7 +411,8 @@ def _add_tools(plot:Figure,
         renderer (GlyphRenderer): Renderer that should call on_click code.
     """
     on_hover_code = "source.data['last_index'] = cb_data.index.indices[0]"
-    plot.add_tools(HoverTool(tooltips=[("Node", "$index")],
+    plot.add_tools(HoverTool(tooltips=[("Index", "$index"),
+                                       ("Name", "@name")],
                              renderers=[nodes_glyph]),
                    HoverTool(tooltips=None,
                              callback=CustomJS(args=dict(source=kw['source']),
@@ -402,12 +451,12 @@ def _get_grid(plot:Figure,
 # ------------------------
 
 
-def _plot_graph(G:nx.Graph,
+def _graph_plot(G:nx.Graph,
                 edges:List[Tuple[int]],
                 cost:float = None,
                 show_all_edges:bool = True,
-                show_labels:bool = True, **kw):
-    """Plot the graph G highlighting the given edges.
+                show_labels:bool = True, **kw) -> GridBox:
+    """Return a plot of the graph G with given edges highlighted.
 
     Args:
         G (nx.Graph): Graph to be plotted.
@@ -415,6 +464,9 @@ def _plot_graph(G:nx.Graph,
         cost (float, optional): Cost to be displayed. Defaults to None.
         show_all_edges (bool, optional): True iff all edges should be shown.
         show_labels (bool, optional): True iff labels should be shown.
+
+    Returns:
+        GridBox: Plot of the graph G with given edges highlighted.
     """
     G = G.copy()
     plot = _blank_plot(G, **kw)
@@ -429,12 +481,13 @@ def _plot_graph(G:nx.Graph,
     if len(edges) > 0:
         xs, ys = _edge_positions(G, edges)
         plot.multi_line(xs=xs, ys=ys, line_cap='round', line_width=LINE_WIDTH,
-                        line_color=TERTIARY_DARK_COLOR, level='image')
+                        line_color=TERTIARY_DARK_COLOR, level=EDGE_LEVEL)
 
     cost_txt = '' if cost is None else ('%.1f' % cost)
     cost = Div(text=cost_txt, width=int(plot.plot_width/2), align='center')
 
-    plot.add_tools(HoverTool(tooltips=[("Node", "$index")],
+    plot.add_tools(HoverTool(tooltips=[("Index", "$index"),
+                                       ("Name", "@name")],
                              renderers=[nodes_glyph]))
     grid = gridplot([[plot],
                      [row(cost)]],
@@ -442,94 +495,56 @@ def _plot_graph(G:nx.Graph,
                     toolbar_location=None,
                     toolbar_options={'logo': None})
 
-    show(grid)
+    return grid
 
 
-def plot_tour(G:nx.Graph, tour:List[int], **kw):
-    """Plot the tour on graph G.
+def tour_plot(G:nx.Graph, tour:List[int], **kw) -> GridBox:
+    """Return a plot of the tour on graph G.
 
     Args:
         G (nx.Graph): Networkx graph.
         tour (List[int]): Tour of the graph.
+
+    Returns:
+        GridBox: Plot of the tour on graph G.
     """
     cost = tour_cost(G, tour)
     edges = [(tour[i], tour[i+1]) for i in range(len(tour)-1)]
-    _plot_graph(G=G, show_all_edges=False, show_labels=False, edges=edges,
-                cost=cost, **kw)
+    return _graph_plot(G=G, show_all_edges=False, show_labels=False,
+                       edges=edges, cost=cost, **kw)
 
 
-def plot_tree(G:nx.Graph,
+def tree_plot(G:nx.Graph,
               tree:List[Tuple[int]],
               show_cost:bool = False, **kw):
-    """Plot the tree on graph G.
+    """Return a plot of the tree on graph G.
 
     Args:
         G (nx.Graph): Networkx graph.
         tree (List[int]): List of edges in the tree.
+        show_cost (bool): True if cost of tree should be shown on plot.
+
+    Returns:
+        GridBox: Plot of the tree on graph G.
     """
-    cost = spanning_tree_cost(G, tree)
-    _plot_graph(G=G, show_all_edges=True, show_labels=True, edges=tree.edges,
-                cost=cost, **kw)
-
-
-def plot_etching_tour(G:nx.Graph, tour:List[int], **kw):
-    """Plot the tour on the etching problem.
-
-    Args:
-        G (nx.Graph): Networkx graph.
-        tour (List[int]): Tour of the graph
-    """
-    # nodes
-    x_start = list(nx.get_node_attributes(G,'x_start').values())
-    x_end = list(nx.get_node_attributes(G,'x_end').values())
-    y_start = list(nx.get_node_attributes(G,'y_start').values())
-    y_end = list(nx.get_node_attributes(G,'y_end').values())
-
-    node_xs = [(G.nodes[i]['x_start'], G.nodes[i]['x_end']) for i in G.nodes]
-    node_ys = [(G.nodes[i]['y_start'], G.nodes[i]['y_end']) for i in G.nodes]
-
-    # tour edges
-    xs = []
-    ys = []
-    for i in range(len(tour)-1):
-        xs.append((G.nodes[tour[i]]['x_end'], G.nodes[tour[i+1]]['x_start']))
-        ys.append((G.nodes[tour[i]]['y_end'], G.nodes[tour[i+1]]['y_start']))
-
-    plot = _blank_plot(G, **kw)
-
-    cost_text = '%.1f' % tour_cost(G, tour)
-    cost = Div(text=cost_text, width=plot.plot_width, align='center')
-    plot.multi_line(xs=node_xs, ys=node_ys, line_color='black', line_width=2)
-    plot.multi_line(xs=xs, ys=ys, line_color='black', line_width=2,
-                    line_dash='dashed')
-    plot.circle(x_start, y_start, size=5, line_color='steelblue',
-                fill_color='steelblue')
-    plot.circle(x_end, y_end, size=5, line_color='#DC0000',
-                fill_color='#DC0000')
-
-    # create layout
-    grid = gridplot([[plot],[cost]],
-                    plot_width=plot.plot_width, plot_height=plot.plot_height,
-                    toolbar_location=None,
-                    toolbar_options={'logo': None})
-
-    show(grid)
-
+    cost = spanning_tree_cost(G, tree) if show_cost else None
+    return _graph_plot(G=G, show_all_edges=True, show_labels=True,
+                       edges=tree, cost=cost, **kw)
 
 # ----------------------------
 # Non-Static Plotting Fuctions
 # ----------------------------
 
 
-def _plot_graph_iterations(G:nx.Graph,
+def _graph_iterations_plot(G:nx.Graph,
                            nodes:List[List[int]] = None,
                            edges:List[List[List[int]]] = None,
                            costs:List[float] = None,
                            tables:List[pd.DataFrame] = None,
                            swaps:List[List[int]] = None,
                            show_all_edges:bool = True,
-                           show_labels:bool = True, **kw):
-    """Plot multiple iterations of a graph that can be toggled through.
+                           show_labels:bool = True, **kw) -> GridBox:
+    """Return a plot of multiple iterations of a graph.
 
     Args:
         G (nx.Graph): Networkx graph.
@@ -540,6 +555,9 @@ def _plot_graph_iterations(G:nx.Graph,
         swaps (List[List[int]]): Edge swap to highlight at every iteration.
         show_all_edges (bool, optional): True iff all edges should be shown.
         show_labels (bool, optional): True iff labels should be shown.
+
+    Returns:
+        GridBox: Plot of multiple iterations of a graph.
     """
     G = G.copy()
     plot = _blank_plot(G, **kw)
@@ -584,7 +602,7 @@ def _plot_graph_iterations(G:nx.Graph,
         edge_subset_src = ColumnDataSource(data={'xs': edge_xs[0],
                                                  'ys': edge_ys[0]})
         plot.multi_line('xs', 'ys', line_color=TERTIARY_DARK_COLOR,
-                        line_width=LINE_WIDTH, level='underlay',
+                        line_width=LINE_WIDTH, level=EDGE_LEVEL,
                         line_cap='round', source=edge_subset_src)
         args_dict['edge_subset_src'] = edge_subset_src
 
@@ -625,10 +643,10 @@ def _plot_graph_iterations(G:nx.Graph,
                                            'swaps_after_y': swaps_after_y[0]})
         plot.multi_line(xs='swaps_before_x', ys='swaps_before_y',
                         line_color=SECONDARY_COLOR, line_width=LINE_WIDTH,
-                        line_cap='round', level='underlay', source=swaps_src)
+                        line_cap='round', level=EDGE_LEVEL, source=swaps_src)
         plot.multi_line(xs='swaps_after_x', ys='swaps_after_y',
                         line_color=SECONDARY_COLOR, line_width=LINE_WIDTH,
-                        line_cap='round', level='underlay',
+                        line_cap='round', level=EDGE_LEVEL,
                         line_dash=[10,12], source=swaps_src)
         args_dict['swaps_src'] = swaps_src
 
@@ -654,7 +672,8 @@ def _plot_graph_iterations(G:nx.Graph,
                          width_policy='fit', sizing_mode='stretch_width')
     prev_button.js_on_click(CustomJS(args=args_dict, code=prev_btn_code))
 
-    plot.add_tools(HoverTool(tooltips=[("Node", "$index")],
+    plot.add_tools(HoverTool(tooltips=[("Index", "$index"),
+                                       ("Name", "@name")],
                              renderers=[nodes_glyph]))
 
     # create layout
@@ -671,27 +690,33 @@ def _plot_graph_iterations(G:nx.Graph,
                     toolbar_location=None,
                     toolbar_options={'logo': None})
 
-    show(grid)
+    return grid
 
 
-def plot_dijkstras(G:nx.Graph, s:int = 0, **kw):
-    """Plot Dijkstra's algorithm running on graph G with source s.
+def dijkstras_plot(G:nx.Graph, s:int = 0, **kw) -> GridBox:
+    """Return plot of Dijkstra's algorithm running on graph G with source s.
 
     Args:
         G (nx.Graph): Networkx graph.
         s (int): Source vertex to run the algorithm from. (Defaults to 0)
+
+    Returns:
+        GridBox: Plot of Dijkstra's algorithm running on graph G with source s.
     """
     nodes, trees, tables = dijkstras(G, s=s, iterations=True)
-    edges = [tree.edges for tree in trees]
-    _plot_graph_iterations(G, nodes=nodes, edges=edges, tables=tables, **kw)
+    return _graph_iterations_plot(G, nodes=nodes, edges=trees,
+                                  tables=tables, **kw)
 
 
-def plot_mst_algorithm(G:nx.Graph, algorithm:str, **kw):
-    """Plot the MST algorithm running on the graph G.
+def mst_algorithm_plot(G:nx.Graph, algorithm:str, **kw) -> GridBox:
+    """Return plot of the given MST algorithm running on the graph G.
 
     Args:
         G (nx.Graph): Networkx graph.
         algorithm (str): {'prims', 'kruskals', 'reverse_kruskals'}
+
+    Returns:
+        GridBox: Plot of the given MST algorithm running on the graph G.
     """
     if algorithm == 'prims':
         trees = prims(G, i=kw['i'], iterations=True)
@@ -699,21 +724,24 @@ def plot_mst_algorithm(G:nx.Graph, algorithm:str, **kw):
         trees = kruskals(G, iterations=True)
     elif algorithm == 'reverse_kruskals':
         trees = reverse_kruskals(G, iterations=True)
-    edges = [tree.edges for tree in trees]
     nodes = []
-    for edge in edges:
-        nodes.append(list(set([item for sublist in edge for item in sublist])))
+    for tree in trees:
+        nodes.append(list(set([item for sublist in tree for item in sublist])))
     costs = [spanning_tree_cost(G, tree) for tree in trees]
-    _plot_graph_iterations(G, nodes=nodes, edges=edges, costs=costs, **kw)
+    return _graph_iterations_plot(G, nodes=nodes, edges=trees,
+                                  costs=costs, **kw)
 
 
-def plot_tsp_heuristic(G:nx.Graph, algorithm:str, **kw):
-    """Plot the TSP heuristic running on G.
+def tsp_heuristic_plot(G:nx.Graph, algorithm:str, **kw) -> GridBox:
+    """Return a plot of the given TSP heuristic running on G.
 
     Args:
         G (nx.Graph): Networkx graph.
         algorithm (str): {'random_neighbor', 'nearest_neighbor',
                           'nearest_insertion', 'furthest_insertion', '2-OPT'}
+
+    Returns:
+        GridBox: Plot of the given TSP heuristic running on G.
     """
     swaps = None
     if algorithm == 'random_neighbor':
@@ -739,10 +767,9 @@ def plot_tsp_heuristic(G:nx.Graph, algorithm:str, **kw):
     for tour in tours:
         edges.append([(tour[i], tour[i+1]) for i in range(len(tour)-1)])
     costs = [tour_cost(G, tour) for tour in tours]
-    _plot_graph_iterations(G, nodes=nodes, edges=edges, costs=costs,
-                           swaps=swaps, show_all_edges=False,
-                           show_labels=False, **kw)
-    return tours[-1]
+    return _graph_iterations_plot(G, nodes=nodes, edges=edges, costs=costs,
+                                  swaps=swaps, show_all_edges=False,
+                                  show_labels=False, **kw)
 
 
 # -----------------------------
@@ -750,11 +777,14 @@ def plot_tsp_heuristic(G:nx.Graph, algorithm:str, **kw):
 # -----------------------------
 
 
-def plot_create_tour(G:nx.Graph, **kw):
-    """Plot in which you can create a tour.
+def create_tour_plot(G:nx.Graph, **kw) -> GridBox:
+    """Return a plot in which you can create a tour.
 
     Args:
         G (nx.Graph): Networkx graph.
+
+    Rerturns:
+        GridBox: Plot in which you can create a tour.
     """
     G = G.copy()
     plot = _blank_plot(G, **kw)
@@ -774,7 +804,7 @@ def plot_create_tour(G:nx.Graph, **kw):
     tour_src = ColumnDataSource(data={'edges_x': [],
                                       'edges_y': []})
     plot.line(x='edges_x', y='edges_y', line_color=TERTIARY_DARK_COLOR,
-              line_cap='round', line_width=LINE_WIDTH, level='image',
+              line_cap='round', line_width=LINE_WIDTH, level=EDGE_LEVEL,
               source=tour_src)
 
     cost, clicked, error_msg = _get_create_divs(plot)
@@ -786,14 +816,17 @@ def plot_create_tour(G:nx.Graph, **kw):
                tour_src=tour_src, cost_matrix=cost_matrix, cost=cost,
                error_msg=error_msg, clicked=clicked)
 
-    show(_get_grid(plot, cost, error_msg, clicked))
+    return _get_grid(plot, cost, error_msg, clicked)
 
 
-def plot_create_spanning_tree(G:nx.Graph, **kw):
-    """Plot in which you can create a spanning tree.
+def create_spanning_tree_plot(G:nx.Graph, **kw) -> GridBox:
+    """Return a plot in which you can create a spanning tree.
 
     Args:
         G (nx.Graph): Networkx graph.
+
+    Returns:
+        GridBox: Plot in which you can create a spanning tree.
     """
     G = G.copy()
     plot = _blank_plot(G, **kw)
@@ -824,15 +857,18 @@ def plot_create_spanning_tree(G:nx.Graph, **kw):
                edges_src=edges_src, cost_matrix=cost_matrix, cost=cost,
                error_msg=error_msg, clicked=clicked)
 
-    show(_get_grid(plot, cost, error_msg, clicked))
+    return _get_grid(plot, cost, error_msg, clicked)
 
 
-def plot_assisted_dijkstras(G, s=0, **kw):
-    """Plot in which the user creates a shortest path tree from s.
+def assisted_dijkstras_plot(G, s=0, **kw) -> GridBox:
+    """Return a plot in which the user creates a shortest path tree from s.
 
     Args:
         G (nx.Graph): Networkx graph.
         s (int): The vertex to generate a shortest path tree from.
+
+    Returns:
+        GridBox: Plot in which the user creates a shortest path tree from s.
     """
     G = G.copy()
     plot = _blank_plot(G, **kw)
@@ -893,15 +929,18 @@ def plot_assisted_dijkstras(G, s=0, **kw):
                     toolbar_location=None,
                     toolbar_options={'logo': None})
 
-    show(grid)
+    return grid
 
 
-def plot_assisted_mst_algorithm(G:nx.Graph, algorithm:str, **kw):
-    """Plot in which the user creates an MST using the given algorithm.
+def assisted_mst_algorithm_plot(G:nx.Graph, algorithm:str, **kw) -> GridBox:
+    """Return a plot in which the user creates an MST with the given algorithm.
 
     Args:
         G (nx.Graph): Networkx graph.
         algorithm (str): {'prims', 'kruskals', 'reverse_kruskals'}
+
+    Returns:
+        GridBox: Plot in which the user creates an MST with given algorithm.
     """
     G = G.copy()
     plot = _blank_plot(G, **kw)
@@ -954,7 +993,7 @@ def plot_assisted_mst_algorithm(G:nx.Graph, algorithm:str, **kw):
         source = ColumnDataSource(data=src_data)
 
     if algorithm == 'reverse_kruskals':
-        cost_text = '%.1f' % spanning_tree_cost(G, Tree(edges=G.edges))
+        cost_text = '%.1f' % spanning_tree_cost(G, G.edges)
         clicked_edges = [str(x).replace(' ', '') for x in list(G.edges)]
         click_txt = '[' + ','.join(clicked_edges) + ']'
         cost, clicked, error_msg = _get_create_divs(plot, cost_txt=cost_text,
@@ -970,4 +1009,4 @@ def plot_assisted_mst_algorithm(G:nx.Graph, algorithm:str, **kw):
                edges_src=edges_src, cost_matrix=cost_matrix, cost=cost,
                error_msg=error_msg, clicked=clicked)
 
-    show(_get_grid(plot, cost, error_msg, clicked))
+    return _get_grid(plot, cost, error_msg, clicked)
